/// Generated by build script from /src/syntax_mapping/builtins/.
pub(crate) static BUILTIN_MAPPINGS: [(
    Lazy<Option<GlobMatcher>>,
    MappingTarget,
); 107usize] = [
    (
        Lazy::new(|| Some(build_matcher_fixed("httpd.conf"))),
        MappingTarget::MapTo("Apache Conf"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/apache2/**/*.conf"))),
        MappingTarget::MapTo("Apache Conf"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/apache2/sites-*/**/*"))),
        MappingTarget::MapTo("Apache Conf"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/httpd/conf/**/*.conf"))),
        MappingTarget::MapTo("Apache Conf"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("**/.aws/credentials"))),
        MappingTarget::MapTo("INI"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("**/.aws/config"))),
        MappingTarget::MapTo("INI"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("**/bat/config"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/letsencrypt/renewal/*.conf"))),
        MappingTarget::MapTo("INI"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("CITATION.cff"))),
        MappingTarget::MapTo("YAML"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("Containerfile"))),
        MappingTarget::MapTo("Dockerfile"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/usr/share/containers/**/*.conf"))),
        MappingTarget::MapTo("TOML"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/containers/**/*.conf"))),
        MappingTarget::MapTo("TOML"),
    ),
    (
        Lazy::new(|| build_matcher_dynamic(
            &[
                MatcherSegment::Env("HOME"),
                MatcherSegment::Text("/.config/containers/**/*.conf"),
            ],
        )),
        MappingTarget::MapTo("TOML"),
    ),
    (
        Lazy::new(|| build_matcher_dynamic(
            &[
                MatcherSegment::Env("XDG_CONFIG_HOME"),
                MatcherSegment::Text("/containers/**/*.conf"),
            ],
        )),
        MappingTarget::MapTo("TOML"),
    ),
    (Lazy::new(|| Some(build_matcher_fixed("*.h"))), MappingTarget::MapTo("C++")),
    (
        Lazy::new(|| Some(build_matcher_fixed(".clang-format"))),
        MappingTarget::MapTo("YAML"),
    ),
    (Lazy::new(|| Some(build_matcher_fixed("*.debdiff"))), MappingTarget::MapTo("Diff")),
    (Lazy::new(|| Some(build_matcher_fixed("*.csproj"))), MappingTarget::MapTo("XML")),
    (Lazy::new(|| Some(build_matcher_fixed("*.vbproj"))), MappingTarget::MapTo("XML")),
    (Lazy::new(|| Some(build_matcher_fixed("*.props"))), MappingTarget::MapTo("XML")),
    (Lazy::new(|| Some(build_matcher_fixed("*.targets"))), MappingTarget::MapTo("XML")),
    (Lazy::new(|| Some(build_matcher_fixed("*.fs"))), MappingTarget::MapTo("F#")),
    (
        Lazy::new(|| Some(build_matcher_fixed("fish_history"))),
        MappingTarget::MapTo("YAML"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.flatpakref"))),
        MappingTarget::MapTo("INI"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.flatpakrepo"))),
        MappingTarget::MapTo("INI"),
    ),
    (
        Lazy::new(|| build_matcher_dynamic(
            &[
                MatcherSegment::Env("XDG_CONFIG_HOME"),
                MatcherSegment::Text("/git/config"),
            ],
        )),
        MappingTarget::MapTo("Git Config"),
    ),
    (
        Lazy::new(|| build_matcher_dynamic(
            &[MatcherSegment::Env("HOME"), MatcherSegment::Text("/.config/git/config")],
        )),
        MappingTarget::MapTo("Git Config"),
    ),
    (
        Lazy::new(|| build_matcher_dynamic(
            &[
                MatcherSegment::Env("XDG_CONFIG_HOME"),
                MatcherSegment::Text("/git/ignore"),
            ],
        )),
        MappingTarget::MapTo("Git Ignore"),
    ),
    (
        Lazy::new(|| build_matcher_dynamic(
            &[MatcherSegment::Env("HOME"), MatcherSegment::Text("/.config/git/ignore")],
        )),
        MappingTarget::MapTo("Git Ignore"),
    ),
    (
        Lazy::new(|| build_matcher_dynamic(
            &[
                MatcherSegment::Env("XDG_CONFIG_HOME"),
                MatcherSegment::Text("/git/attributes"),
            ],
        )),
        MappingTarget::MapTo("Git Attributes"),
    ),
    (
        Lazy::new(|| build_matcher_dynamic(
            &[
                MatcherSegment::Env("HOME"),
                MatcherSegment::Text("/.config/git/attributes"),
            ],
        )),
        MappingTarget::MapTo("Git Attributes"),
    ),
    (Lazy::new(|| Some(build_matcher_fixed("*.jsonl"))), MappingTarget::MapTo("JSON")),
    (Lazy::new(|| Some(build_matcher_fixed("*.jsonc"))), MappingTarget::MapTo("JSON")),
    (Lazy::new(|| Some(build_matcher_fixed("*.jsonld"))), MappingTarget::MapTo("JSON")),
    (Lazy::new(|| Some(build_matcher_fixed("*.geojson"))), MappingTarget::MapTo("JSON")),
    (Lazy::new(|| Some(build_matcher_fixed("*.ndjson"))), MappingTarget::MapTo("JSON")),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.ksh"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.kshrc"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/kubernetes/*.conf"))),
        MappingTarget::MapTo("YAML"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/var/spool/mail/*"))),
        MappingTarget::MapTo("Email"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/var/mail/*"))),
        MappingTarget::MapTo("Email"),
    ),
    (Lazy::new(|| Some(build_matcher_fixed("*.mkd"))), MappingTarget::MapTo("Markdown")),
    (Lazy::new(|| Some(build_matcher_fixed("*.mill"))), MappingTarget::MapTo("Scala")),
    (
        Lazy::new(|| Some(build_matcher_fixed("nginx.conf"))),
        MappingTarget::MapTo("nginx"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("mime.types"))),
        MappingTarget::MapTo("nginx"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/nginx/**/*.conf"))),
        MappingTarget::MapTo("nginx"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/nginx/sites-*/**/*"))),
        MappingTarget::MapTo("nginx"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("flake.lock"))),
        MappingTarget::MapTo("JSON"),
    ),
    (Lazy::new(|| Some(build_matcher_fixed("*.nse"))), MappingTarget::MapTo("Lua")),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/os-release"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/usr/lib/os-release"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/initrd-release"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(
            build_matcher_fixed("/usr/lib/extension-release.d/extension-release.*"),
        )),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/pacman.conf"))),
        MappingTarget::MapTo("INI"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/usr/share/libalpm/hooks/*.hook"))),
        MappingTarget::MapTo("INI"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/pacman.d/hooks/*.hook"))),
        MappingTarget::MapTo("INI"),
    ),
    (
        Lazy::new(|| build_matcher_dynamic(&[MatcherSegment::Env("PARU_CONF")])),
        MappingTarget::MapTo("INI"),
    ),
    (Lazy::new(|| Some(build_matcher_fixed("paru.conf"))), MappingTarget::MapTo("INI")),
    (
        Lazy::new(|| Some(
            build_matcher_fixed(
                "**/containers/systemd/**/*.{container,volume,network,kube,image,build,pod,artifact}",
            ),
        )),
        MappingTarget::MapTo("INI"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.pac"))),
        MappingTarget::MapTo("JavaScript (Babel)"),
    ),
    (Lazy::new(|| Some(build_matcher_fixed("*.ron"))), MappingTarget::MapTo("Rust")),
    (Lazy::new(|| Some(build_matcher_fixed("*.sarif"))), MappingTarget::MapTo("JSON")),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/profile"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("bashrc"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.bashrc"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("bash_profile"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.bash_profile"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("bash_login"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.bash_login"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("bash_logout"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.bash_logout"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("zshrc"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.zshrc"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("zprofile"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.zprofile"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("zlogin"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.zlogin"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("zlogout"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.zlogout"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("zshenv"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.zshenv"))),
        MappingTarget::MapTo("Bourne Again Shell (bash)"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("**/.ssh/config"))),
        MappingTarget::MapTo("SSH Config"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("/var/log/dmesg"))),
        MappingTarget::MapTo("Syslog"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("**/systemd/**/*.conf"))),
        MappingTarget::MapTo("INI"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("**/systemd/**/*.example"))),
        MappingTarget::MapTo("INI"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.automount"))),
        MappingTarget::MapTo("INI"),
    ),
    (Lazy::new(|| Some(build_matcher_fixed("*.device"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.dnssd"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.link"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.mount"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.netdev"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.network"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.nspawn"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.path"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.service"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.scope"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.slice"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.socket"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.swap"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.target"))), MappingTarget::MapTo("INI")),
    (Lazy::new(|| Some(build_matcher_fixed("*.timer"))), MappingTarget::MapTo("INI")),
    (
        Lazy::new(|| Some(build_matcher_fixed("/etc/wireguard/*.conf"))),
        MappingTarget::MapTo("INI"),
    ),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.conf"))),
        MappingTarget::MapExtensionToUnknown,
    ),
    (Lazy::new(|| Some(build_matcher_fixed("build"))), MappingTarget::MapToUnknown),
    (Lazy::new(|| Some(build_matcher_fixed("rails"))), MappingTarget::MapToUnknown),
    (Lazy::new(|| Some(build_matcher_fixed("*.xsh"))), MappingTarget::MapTo("Python")),
    (
        Lazy::new(|| Some(build_matcher_fixed("*.xonshrc"))),
        MappingTarget::MapTo("Python"),
    ),
];
